let data;
let AXIS_WIDTH;
let AXIS_HEIGHT;
let countries;

function preload() {
  data = loadTable('https://gist.githubusercontent.com/colmpat/43366b017598cc4d49a6bbe0404c743a/raw/fbd18baebd690ed9d8fb6ee7435d27bc26abd5bf/Europe-Power-Sector-2020-Heatmap.csv', 'csv', 'header');
}

function setup() {
  createCanvas(960, 525);
  angleMode(DEGREES);
  rectMode(CENTER);

  AXIS_WIDTH = width - 120;
  AXIS_HEIGHT = height - 125;

  countries = [];
  for(let country of data.getColumn(1)) {
    if(!countries.includes(country)) {
      countries.push(country);
    }
  }

}

function draw() {
  background(250,250,255);

  title("Average Share of Production for Renewables per Country, from 2000 to 2020", 30);
  drawAxis(95, 25, 60, 65);
  labelAxis(data.columns[0], 95, 0);
  drawBoxes(95, 65, 4);
  drawLegend(60, 20, 15, 0);
  noLoop();
}

function title(title, inset) {
  textSize(22); fill(37,48,49);
  text(title, inset, inset);
}

function drawAxis(xInsetL, xInsetR, yInsetT, yInsetB) {
  stroke(37,48,49); strokeWeight(3);
  //top left to bottom left
  line(xInsetL, yInsetT, xInsetL, height - yInsetB);
  //bottom left to bottom right
  line(xInsetL, height - yInsetB, width - xInsetR, height - yInsetB);
}

function labelAxis(xLabel, xInset, yInset) {
  textAlign(CENTER, BOTTOM); textSize(22); noStroke();
  //label x axis
  text(xLabel, (AXIS_WIDTH / 2) + xInset, height - yInset);
}

function drawBoxes(xInset, yInset, spacing) {
  rectMode(CENTER);
  noStroke();
  let sideLength = (AXIS_HEIGHT - spacing * (countries.length + 1)) / countries.length;
  let runningY = height - yInset - sideLength / 2.0 - spacing;
  for(let country of countries) {
    let values = [];
    data.getRows().forEach(row => {
      if(row.get(1) === country) {
        values[values.length] = row.get(2);
      }
    });
    //label Country
    //draw boxes
    let runningX = xInset + spacing + sideLength / 2.0;
    for(let value of values) {
      fill(getColor(value));
      square(runningX, runningY, sideLength);
      runningX += sideLength + spacing;
    }

    textSize(16); fill(37,48,49); textAlign(RIGHT, CENTER);
    text(country === "United Kingdom" ? "U.K." : country, xInset - 8, runningY);

    runningY -= spacing + sideLength;
  }
  let runningX = xInset + spacing + sideLength / 2.0;
  textAlign(CENTER, TOP); textSize(18);
  let usedYears = [];
  for(let year of data.getColumn(0)) {
    if(!usedYears.includes(year)) {
      if(year % 5 === 0) {
        text(year, runningX, height - yInset + 4);
      }
      runningX += sideLength + spacing;
      usedYears.push(year);
    }
  }
}

function drawLegend(w, h, xInset, yInset) {
  for(let i = 0; i < w; i++) {
    stroke(getColor(100 * i / w)); strokeWeight(1);
    line(width - xInset - w + i, height - yInset - h, width - xInset - w + i, height - yInset);
  }
  textSize(14); fill(37,48,49); textAlign(LEFT, BOTTOM); noStroke();
  text("0", width - xInset - w, height - yInset - h - 3);
  textAlign(RIGHT, BOTTOM);
  text("100", width - xInset, height - yInset - h - 3);

  textSize(17); textAlign(RIGHT, CENTER);
  text("Avg. Share of Production (%)", width - xInset - w - 4, height - yInset - h / 2.0);
}


function getColor(shareOfProduction) {
  let dark = color(11,29,20);
  let green = color(64, 160,104);
  let light = color(167,220,190);
  let percent = shareOfProduction / 100.0;
  if(percent < 0.5) {
    return lerpColor(dark, green, percent * 2.0);
  } else {
    return lerpColor(green, light, (percent - 0.5) * 2.0);
  }

}
